# This product is licensed under the GNU Lesser General Public License:
#
# ATManagedFile  |  Copyright(C), 2004, Enfold Systems, LLC
#
# Enfold Systems, LLC
# 4500 Montrose Blvd., Suite D
# Houston, Texas 77006 USA
# p. +1 713.942.2377 | f. +1 832.201.8856
# www.enfoldsystems.com
# info@enfoldsystems.com
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# For a full copy of the license, please see the included 'LGPL.txt' file.


from Products.Archetypes.public import *
from Products.CMFManagedFile.content import ManagedFileMixin, deleteHook
from Products.CMFManagedFile.content import addManagedFile

from schema import managed_file_schema
from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName

from Products.CMFManagedFile.config import *
from ComputedAttribute import ComputedAttribute

def addATManagedFile(*args, **kw):
    kw['factory'] = ATManagedFile
    return addManagedFile(*args, **kw)

class ATManagedFile(BaseContent, ManagedFileMixin):
    """ Archetypes-based Managed File
    """

    meta_type = portal_type = archetype_name = 'AT Managed File'

    schema = BaseContent.schema + managed_file_schema

    security = ClassSecurityInfo()

    _properties = tuple(ManagedFileMixin._properties)

    def __init__(self, id, title='', description='', filesystempath=''):
        ManagedFileMixin.__init__(self, id, title, description, filesystempath)

    def _notifyOfCopyTo(self, container, op=0):
        ManagedFileMixin._notifyOfCopyTo(self, container, op)
        BaseContent._notifyOfCopyTo(self, container, op)

    def manage_beforeDelete(self, item, container):
        ManagedFileMixin.manage_beforeDelete(self, item, container)
        BaseContent.manage_beforeDelete(self, item, container)

    security.declareProtected(ModifyPortalContent, 'setRepository')
    def setRepository(self, value, **kwargs):
        ManagedFileMixin.edit(self, repository=value)

    security.declareProtected(View, 'getSize')
    getSize = ManagedFileMixin.get_size

    security.declareProtected(View, 'get_size')
    get_size = getSize

    security.declareProtected(View, 'at_download')
    at_download = ManagedFileMixin.download

    security.declareProtected(View, 'index_html')
    def index_html(self):
        """ return """
        return ManagedFileMixin.download(self)

    security.declareProtected(View, 'file')
    def file(self):
        """ fool archetypes """
        return self
    file = ComputedAttribute(file, 1)

    security.declareProtected(ModifyPortalContent, 'update')
    def update(self, **kwargs):
        """Alias for edit()
        """
        self.edit(**kwargs)

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, **kwargs):
        """Call edit from ManagedFileMixin and BaseContent
        """
        ManagedFileMixin.edit(self,
                              precondition=kwargs.get('precondition', ''),
                              file=kwargs.get('file', ''),
                              repository=kwargs.get('repository', ''),
                              delete_policy=kwargs.get('delete_policy', ''))
        BaseContent.update(self, **kwargs)

registerType(ATManagedFile)
